from pathlib import Path
from enum import Enum
from pydantic import BaseModel, Extra
from typing import List, Dict, Set, Union, Any
import httpx
from nonebot import get_driver
from nonebot import logger
try:
    import ujson as json
except ModuleNotFoundError:
    import json

class PluginConfig(BaseModel, extra=Extra.ignore):
    what2eat_path: Path = Path(__file__).parent / "resource"
    use_preset_menu: bool = False
    use_preset_greetings: bool = False
    eating_limit: int = 5
    greeting_groups_id: Set[str] = set()
    superusers: Set[str] = set()
    
class Meals(Enum):
    BREAKFAST = ["breakfast", "早餐", "早饭"]
    LUNCH = ["lunch", "午餐", "午饭", "中餐"]
    SNACK = ["snack", "摸鱼", "下午茶", "饮茶"]
    DINNER = ["dinner", "晚餐", "晚饭"]
    MIDNIGHT = ["midnight", "夜宵", "宵夜"]
    
driver = get_driver()
what2eat_config: PluginConfig = PluginConfig.parse_obj(driver.config.dict())

class DownloadError(Exception):
    def __init__(self, msg):
        self.msg = msg
        
    def __str__(self):
        return self.msg
    
async def download_url(url: str) -> Union[Any, None]:
    async with httpx.AsyncClient() as client:
        for i in range(3):
            try:
                response = await client.get(url)
                if response.status_code != 200:
                    continue
                return response.json()
            except Exception as e:
                logger.warning(f"Error occured when downloading {url}, {i+1}/3: {e}")
    
    logger.warning(f"Abort downloading")
    return None

def save_json(_file: Path, _data: Any) -> None:
    with open(_file, 'w', encoding='utf-8') as f:
        json.dump(_data, f, ensure_ascii=False, indent=4)
        
def write_init_keys(_file: Path, _name: str) -> None:
    '''
        Write ALL the initial keys
    '''
    _data = {}
    if _name == "eating.json":
        _data["basic_food"] = []
        _data["group_food"] = {}
        _data["count"] = {}
        save_json(_file, _data)
    else:
        for meal in Meals:
            if meal not in _data:
                _data[meal.value[0]] = []
        
        _data["groups_id"] = {}
        save_json(_file, _data)
               
async def download_file(_file: Path, _name: str) -> None:
    _url = "https://raw.fastgit.org/MinatoAquaCrews/nonebot_plugin_what2eat/beta/nonebot_plugin_what2eat/resource/"
    url = _url + _name
    
    resp = await download_url(url)
    if resp is None:
        # Write initial keys in file when download failed
        write_init_keys(_file, _name)
        logger.info(f"Write inital keys in file {_name}")
    else:
        save_json(_file, resp)
        logger.info(f"Get file {_name} from repo")
        
@driver.on_startup
async def what2eat_check() -> None:
    '''
        Check the path, gnerate the groups id saved in eating.json
        If needed, download the preset menu and greetings
    '''
    if not what2eat_config.what2eat_path.exists():
        what2eat_config.what2eat_path.mkdir(parents=True, exist_ok=True)
    
    '''
        If eating.json doesn't exist or eating.json exists but f.get["basic_food"] doesn't exist and USE_PRESET_MENU is True, download
        If USE_PRESET_MENU is False, break
    '''
    eating_json: Path = what2eat_config.what2eat_path / "eating.json"
    if what2eat_config.use_preset_menu:
        if not eating_json.exists():
            await download_file(eating_json, "eating.json")
        else:
            # Exists then check the keys
            with eating_json.open("r", encoding="utf-8") as f:
                _f: Dict[str, Union[List[str], Dict[str, Union[Dict[str, List[int]], List[str]]]]] = json.load(f)
                if not _f.get("basic_food", False):
                    _f.update({"basic_food", []})
                
                if not _f.get("group_food", False):
                    _f.update({"group_food", {}})
                    
                if not _f.get("count", False):
                    _f.update({"count", {}})
                
            save_json(eating_json, _f)
    
    '''
        If greetings.json doesn't exist or greetings.json exists but ... ALL doesn't exist and USE_PRESET_greetings is True, download
        If USE_PRESET_greetings is False, break
    '''
    greetings_json: Path = what2eat_config.what2eat_path / "greetings.json"
    if what2eat_config.use_preset_greetings:
        if not greetings_json.exists():
            await download_file(greetings_json, "greetings.json")    
        else:
            # Exists then check the keys
            with greetings_json.open("r", encoding="utf-8") as f:
                _f: Dict[str, Union[List[str], Dict[str, bool]]] = json.load(f)
                for meal in Meals:
                    if _f.get(meal.value[0], False):
                        _f.update({meal.value[0], []})
                
                if not _f.get("groups_id", False):
                    _f["groups_id"] = {}
            
            save_json(greetings_json, _f)
    
    # Save groups id in greetings.json if len > 0
    if len(what2eat_config.greeting_groups_id) > 0:
        with greetings_json.open("r", encoding="utf-8") as f:
            _f: Dict[str, Union[List[str], Dict[str, bool]]] = json.load(f)
            
            # Update the default groups    
            if not _f.get("groups_id", False):
                _f["groups_id"] = {}
                        
            for gid in what2eat_config.greeting_groups_id:
                _f["groups_id"].update({gid: True})
        
        save_json(greetings_json, _f)

__all__ = [
    Meals, what2eat_config, save_json
]


from typing import Coroutine, Any
from nonebot import on_command, on_regex
from nonebot.typing import T_State
from nonebot.permission import SUPERUSER
from nonebot.adapters.onebot.v11 import GROUP, GROUP_ADMIN, GROUP_OWNER, Message, MessageEvent, GroupMessageEvent
from nonebot.params import Depends, Arg, ArgStr, CommandArg, RegexMatched
from nonebot.matcher import Matcher
from nonebot import logger
from nonebot_plugin_apscheduler import scheduler
from .config import Meals
from .data_source import eating_manager

__what2eat_version__ = "v0.3.1"
__what2eat_notes__ = f'''
今天吃什么？ {__what2eat_version__}
[xx吃xx]    问bot恰什么
[添加 xx]   添加菜品至群菜单
[移除 xx]   从菜单移除菜品
[加菜 xx]   添加菜品至基础菜单
[菜单]       查看群菜单
[基础菜单]  查看基础菜单
[开启/关闭小助手] 开启/关闭吃饭小助手
[添加/删除问候 时段 问候语] 添加/删除吃饭小助手问候语'''.strip()

what2eat = on_regex(r"^(今天|[早中午晚][上饭餐午]|早上|夜宵|今晚)吃(什么|啥|点啥)(帮助)?$", priority=15)
group_add = on_command("添加", permission=SUPERUSER | GROUP_ADMIN | GROUP_OWNER, priority=15, block=True)
group_remove = on_command("移除", permission=SUPERUSER | GROUP_ADMIN | GROUP_OWNER, priority=15, block=True)
basic_add = on_command("加菜", permission=SUPERUSER, priority=15, block=True)
show_group_menu = on_command("菜单", aliases={"群菜单", "查看菜单"}, permission=GROUP, priority=15, block=True)
show_basic_menu = on_command("基础菜单", permission=GROUP, priority=15, block=True)

greeting_on = on_command("开启小助手", aliases={"启用小助手"}, permission=SUPERUSER | GROUP_ADMIN | GROUP_OWNER, priority=15, block=True)
greeting_off = on_command("关闭小助手", aliases={"禁用小助手"}, permission=SUPERUSER | GROUP_ADMIN | GROUP_OWNER, priority=15, block=True)
add_greeting = on_command("添加问候", aliases={"添加问候语"}, permission=SUPERUSER | GROUP_ADMIN | GROUP_OWNER, priority=15, block=True)
remove_greeting = on_command("删除问候", aliases={"删除问候语", "移除问候", "移除问候语"}, permission=SUPERUSER | GROUP_ADMIN | GROUP_OWNER, priority=15, block=True)

@what2eat.handle()
async def _(event: MessageEvent, args: str = RegexMatched()):
    if args[-2:] == "帮助":
        await what2eat.finish(__what2eat_notes__)
    
    msg = eating_manager.get2eat(event)
    await what2eat.finish(msg)

@group_add.handle()
async def _(event: GroupMessageEvent, args: Message = CommandArg()):
    args = args.extract_plain_text().strip().split()
    if not args:
        await group_add.finish("还没输入你要添加的菜品呢~")
    elif args and len(args) == 1:
        new_food = args[0]
    else:
        await group_add.finish("添加菜品参数错误~")
    
    msg = eating_manager.add_group_food(new_food, event)

    await group_add.finish(msg)

@basic_add.handle()
async def _(args: Message = CommandArg()):
    args = args.extract_plain_text().strip().split()
    if not args:
        await basic_add.finish("还没输入你要添加的菜品呢~")
    elif args and len(args) == 1:
        new_food = args[0]
    else:
        await basic_add.finish("添加菜品参数错误~")

    msg = eating_manager.add_basic_food(new_food)

    await basic_add.finish(msg)

@group_remove.handle()
async def _(event: GroupMessageEvent, args: Message = CommandArg()):
    args = args.extract_plain_text().strip().split()
    if not args:
        await group_remove.finish("还没输入你要移除的菜品呢~")
    elif args and len(args) == 1:
        food_to_remove = args[0]
    else:
        await group_remove.finish("移除菜品参数错误~")

    msg = eating_manager.remove_food(event, food_to_remove)

    await group_remove.finish(msg)

@show_group_menu.handle()
async def _(event: GroupMessageEvent):
    gid = str(event.group_id)
    msg = eating_manager.show_group_menu(gid)
    await show_group_menu.finish(msg)

@show_basic_menu.handle()
async def _(matcher: Matcher):
    msg = eating_manager.show_basic_menu()
    await matcher.finish(msg)

# ------------------------- Greetings -------------------------
@greeting_on.handle()
async def _(event: GroupMessageEvent):
    gid = str(event.group_id)
    eating_manager.update_groups_on(gid, True)
    await greeting_on.finish("已开启吃饭小助手~")

@greeting_off.handle()
async def _(event: GroupMessageEvent):
    gid = str(event.group_id)
    eating_manager.update_groups_on(gid, False)
    await greeting_off.finish("已关闭吃饭小助手~")

def parse_greeting() -> Coroutine[Any, Any, None]:
    '''
        Parser the greeting input from user then store in state["greeting"]
    '''
    async def _greeting_parser(matcher: Matcher, state: T_State, input_arg: Message = Arg("greeting")) -> None:
        if input_arg.extract_plain_text() == "取消":
            await matcher.finish("操作已取消")
        else:
            state["greeting"] = input_arg
    
    return _greeting_parser

def parse_meal() -> Coroutine[Any, Any, None]:
    '''
        Parser the meal input from user then store in state["meal"]. If illigal, reject it
    '''
    async def _meal_parser(matcher: Matcher, state: T_State, input_arg: str = ArgStr("meal")) -> None:
        if input_arg == "取消":
            await matcher.finish("操作已取消")
            
        res = eating_manager.which_meals(input_arg)
        if res is None:
            await matcher.reject_arg("meal", "输入时段不合法")
        else:
            state["meal"] = res
    
    return _meal_parser

def parse_index() -> None:
    '''
        Parser the index of greeting to be removed input from user then store in state["index"]
    '''
    async def _index_parser(matcher: Matcher, state: T_State, input_arg: str = ArgStr("index")) -> None:
        try:
            arg2int = int(input_arg)
        except ValueError:
            await matcher.reject_arg("index", "输入序号不合法")
        
        if arg2int == 0:
            await matcher.finish("操作已取消")
        else:
            state["index"] = arg2int
    
    return _index_parser
        
@add_greeting.handle()
async def _(matcher: Matcher, args: Message = CommandArg()):
    args = args.extract_plain_text().strip().split()
    if args and len(args) <= 2:
        res = eating_manager.which_meals(args[0])
        if isinstance(res, Meals):
            matcher.set_arg("meal", args[0])
            if len(args) == 2:
                matcher.set_arg("greeting", args[1])
    
@remove_greeting.handle()
async def _(matcher: Matcher, args: Message = CommandArg()):
    args = args.extract_plain_text().strip().split()
    if args:
        res = eating_manager.which_meals(args[0])
        if isinstance(res, Meals):
            matcher.set_arg("meal", args[0])
    
@add_greeting.got(
    "meal",
    prompt="请输入添加问候语的时段，可选：早餐/午餐/摸鱼/晚餐/夜宵，输入取消以取消操作",
    parameterless=[Depends(parse_meal())]
)
async def handle_skip():
    add_greeting.skip()

@add_greeting.got(
    "greeting", 
    prompt="请输入添加的问候语，输入取消以取消操作",
    parameterless=[Depends(parse_greeting())]
)
async def handle_add_greeting(state: T_State, greeting: Message = Arg()):
    meal = state["meal"]
    # Not support for text + image greeting, just extract the plain text
    msg = eating_manager.add_greeting(meal, greeting.extract_plain_text())
    await add_greeting.finish(msg)

@remove_greeting.got(
    "meal",
    prompt="请输入删除问候语的时段，可选：早餐/午餐/摸鱼/晚餐/夜宵，输入取消以取消操作",
    parameterless=[Depends(parse_meal())]
)
async def handle_show_greetings(meal: Meals = Arg()):
    msg = eating_manager.show_greetings(meal)
    await remove_greeting.send(msg)
    
@remove_greeting.got(
    "index",
    prompt="请输入删除的问候语序号，输入0以取消操作",
    parameterless=[Depends(parse_index())]
)
async def handle_remove_greeting(state: T_State, index: int = Arg()):
    meal = state["meal"]
    msg = eating_manager.remove_greeting(meal, index)
    await remove_greeting.finish(msg)

# ------------------------- Schedulers -------------------------
# 重置吃什么次数，包括夜宵
@scheduler.scheduled_job("cron", hour="6,11,17,22", minute=0, misfire_grace_time=60)
async def _():
    eating_manager.reset_count()
    logger.info("今天吃什么次数已刷新")

# 早餐提醒
@scheduler.scheduled_job("cron", hour=7, minute=0, misfire_grace_time=60)
async def time_for_breakfast():
    await eating_manager.do_greeting(Meals.BREAKFAST)
    logger.info(f"已群发早餐提醒")

# 午餐提醒
@scheduler.scheduled_job("cron", hour=12, minute=0, misfire_grace_time=60)
async def time_for_lunch():
    await eating_manager.do_greeting(Meals.LUNCH)
    logger.info(f"已群发午餐提醒")

# 下午茶/摸鱼提醒
@scheduler.scheduled_job("cron", hour=15, minute=0, misfire_grace_time=60)
async def time_for_snack():
    await eating_manager.do_greeting(Meals.SNACK)
    logger.info(f"已群发摸鱼提醒")

# 晚餐提醒
@scheduler.scheduled_job("cron", hour=18, minute=0, misfire_grace_time=60)
async def time_for_dinner():
    await eating_manager.do_greeting(Meals.DINNER)
    logger.info(f"已群发晚餐提醒")

# 夜宵提醒
@scheduler.scheduled_job("cron", hour=22, minute=0, misfire_grace_time=60)
async def time_for_midnight():
    await eating_manager.do_greeting(Meals.MIDNIGHT)
    logger.info(f"已群发夜宵提醒")


from nonebot.adapters.onebot.v11 import MessageEvent, GroupMessageEvent, PrivateMessageEvent, MessageSegment
from nonebot.adapters.onebot.exception import ActionFailed
from nonebot import get_bot, logger
import random
from pathlib import Path
from enum import Enum
from typing import Optional, Union, List, Dict
from .config import Meals, what2eat_config
from .utils import do_compatible
try:
    import ujson as json
except ModuleNotFoundError:
    import json
    
class FoodLoc(Enum):
    IN_BASIC = "In_Basic"
    IN_GROUP = "In_Group"
    NOT_EXISTS = "Not_Exists"

class EatingManager:
    def __init__(self):
        self._init_ok: bool = False
        self._eating: Dict[str, Union[List[str], Dict[str, Union[Dict[str, List[int]], List[str]]]]] = {}
        self._greetings: Dict[str, Union[List[str], Dict[str, bool]]] = {}
        
        self._eating_json: Path = what2eat_config.what2eat_path / "eating.json"
        self._greetings_json: Path = what2eat_config.what2eat_path / "greetings.json"
        
        '''
            Compatible work will be deprecated in next version
        '''
        do_compatible(self._eating_json, self._greetings_json)
        
    def _init_json(self) -> None:
        self._init_ok = True
        with open(self._eating_json, 'r', encoding='utf-8') as f:
            self._eating = json.load(f)
            
        with open(self._greetings_json, 'r', encoding='utf-8') as f:
            self._greetings = json.load(f)
    
    def _init_data(self, gid: str, uid: str) -> None:
        '''
            初始化用户信息
        '''
        if not self._init_ok:
            self._init_json()
            
        if gid not in self._eating["group_food"]:
            self._eating["group_food"][gid] = []
        if gid not in self._eating["count"]:
            self._eating["count"][gid] = {}
        if uid not in self._eating["count"][gid]:
            self._eating["count"][gid][uid] = 0

    def get2eat(self, event: MessageEvent) -> MessageSegment:
        '''
            今天吃什么
        '''
        if not self._init_ok:
            self._init_json()
            
        if isinstance(event, PrivateMessageEvent):
            if len(self._eating["basic_food"]) == 0:
                return MessageSegment.text("还没有菜单呢，就先饿着肚子吧，请[添加 菜名]🤤")
            else:
                return MessageSegment.text("建议") + MessageSegment.text(random.choice(self._eating["basic_food"]))
            
        uid = str(event.user_id)
        gid = str(event.group_id)
        food_list: List[str] = []

        self._init_data(gid, uid)
        if not self._eating_check(gid, uid):
            return random.choice(
                [
                    "你今天已经吃得够多了！",
                    "吃这么多的吗？",
                    "害搁这吃呢？不工作的吗？",
                    "再吃肚子就要爆炸咯~",
                    "你是米虫吗？今天碳水要爆炸啦！"
                ]
            )
        else:
            # basic_food and group_food both are EMPTY
            if len(self._eating["basic_food"]) == 0 and len(self._eating["group_food"][gid]) == 0:
                return MessageSegment.text("还没有菜单呢，就先饿着肚子吧，请[添加 菜名]🤤")
            
            food_list = self._eating["basic_food"].copy()
            if len(self._eating["group_food"][gid]) > 0:
                food_list.extend(self._eating["group_food"][gid])

            # Even a food maybe in basic AND group menu, probability of it is doubled
            msg = MessageSegment.text("建议") + MessageSegment.text(random.choice(food_list))
            self._eating["count"][gid][uid] += 1
            self._save()

            return msg

    def _is_food_exists(self, _food: str, gid: Optional[str]) -> FoodLoc:
        '''
            检查菜品是否存在
        '''
        for food in self._eating["basic_food"]:
            if food == _food:
                return FoodLoc.IN_BASIC
            
        if isinstance(gid, str):
            if gid in self._eating["group_food"]:
                for food in self._eating["group_food"][gid]:
                    if food == _food:
                        return FoodLoc.IN_GROUP
        
        return FoodLoc.NOT_EXISTS

    def _eating_check(self, gid: str, uid: str) -> bool:
        '''
            检查是否吃饱
        '''
        return False if self._eating["count"][gid][uid] >= what2eat_config.eating_limit else True

    def add_group_food(self, new_food: str, event: GroupMessageEvent) -> MessageSegment:
        '''
            添加至群菜单中 GROUP_ADMIN | GROUP_OWNER 权限
        '''
        uid = str(event.user_id)
        gid = str(event.group_id)
        msg: MessageSegment = ""

        self._init_data(gid, uid)
        status: FoodLoc = self._is_food_exists(new_food, gid)
        
        if status == FoodLoc.IN_BASIC:
            msg = MessageSegment.text(f"{new_food} 已在基础菜单中~")
        elif status == FoodLoc.IN_GROUP:
            msg = MessageSegment.text(f"{new_food} 已在群特色菜单中~")
        else:
            self._eating["group_food"][gid].append(new_food)
            self._save()
            msg = MessageSegment.text(f"{new_food} 已加入群特色菜单~")
        
        return msg

    def add_basic_food(self, new_food: str) -> MessageSegment:
        '''
            添加至基础菜单 SUPERUSER 权限
        '''
        if not self._init_ok:
            self._init_json()
        status: FoodLoc = self._is_food_exists(new_food)
        
        if status == FoodLoc.IN_BASIC:
            msg = MessageSegment.text(f"{new_food} 已在基础菜单中~")
            
        elif status == FoodLoc.NOT_EXISTS:
            self._eating["basic_food"].append(new_food)
            self._save()
            msg = MessageSegment.text(f"{new_food} 已加入基础菜单~")
        
        return msg

    def remove_food(self, event: GroupMessageEvent, food_to_remove: str) -> MessageSegment:
        '''
            从基础菜单移除，需SUPERUSER 权限
            从群菜单中移除，需GROUP_ADMIN | GROUP_OWNER 权限
        '''
        uid = str(event.user_id)
        gid = str(event.group_id)
        msg: MessageSegment = ""
        
        self._init_data(gid, uid)
        status: FoodLoc = self._is_food_exists(food_to_remove, gid)

        if status == FoodLoc.IN_GROUP:
            self._eating["group_food"][gid].remove(food_to_remove)
            self._save()
            msg = MessageSegment.text(f"{food_to_remove} 已从群菜单中删除~")
            
        elif status == FoodLoc.IN_BASIC:
            if uid not in what2eat_config.superusers:
                msg = MessageSegment.text(f"{food_to_remove} 在基础菜单中，非超管不可操作哦~")
            else:
                self._eating["basic_food"].remove(food_to_remove)
                self._save()
                msg = MessageSegment.text(f"{food_to_remove} 已从基础菜单中删除~")   
        else:
            msg = MessageSegment.text(f"{food_to_remove} 不在菜单中哦~")
        
        return msg
    
    def reset_count(self) -> None:
        '''
            重置三餐 eating times
        '''
        if not self._init_ok:
            self._init_json()
        for gid in self._eating["count"]:
            for uid in self._eating["count"][gid]:
                self._eating["count"][gid][uid] = 0
        
        self._save()

    # ------------------------- Menu -------------------------
    def show_group_menu(self, gid: str) -> MessageSegment:
        if not self._init_ok:
            self._init_json()
            
        msg: MessageSegment = ""
        
        if gid not in self._eating["group_food"]:
            self._eating["group_food"][gid] = []
            
        if len(self._eating["group_food"][gid]) > 0:
            msg += MessageSegment.text("---群特色菜单---")
            for food in self._eating["group_food"][gid]:
                msg += MessageSegment.text(f"\n{food}")
            
            return msg
        
        return MessageSegment.text("还没有群特色菜单呢，请[添加 菜名]🤤")

    def show_basic_menu(self) -> MessageSegment:
        if not self._init_ok:
            self._init_json()

        msg: MessageSegment = ""

        if len(self._eating["basic_food"]) > 0:
            msg += MessageSegment.text("---基础菜单---")
            for food in self._eating["basic_food"]:
                msg += MessageSegment.text(f"\n{food}")
            
            return msg
        
        return MessageSegment.text("还没有基础菜单呢，请[添加 菜名]🤤")

    # ------------------------- greetings -------------------------
    def update_groups_on(self, gid: str, new_state: bool) -> None:
        '''
            Turn on/off greeting tips in group
        '''
        if not self._init_ok:
            self._init_json()
        if new_state:
            if gid not in self._greetings["groups_id"]:
                self._greetings["groups_id"].update({gid: True})
        else:
            if gid in self._greetings["groups_id"]:
                self._greetings["groups_id"].update({gid: False})
        
        self._save()
        
    def which_meals(self, input_cn: str) -> Union[Meals, None]:
        '''
            Judge which meals is user's input indicated
        '''
        if not self._init_ok:
            self._init_json()
            
        for meal in Meals:
            if input_cn in meal.value:
                return meal
        else:
            return None

    def add_greeting(self, meal: Meals, greeting: str) -> MessageSegment:
        '''
            添加某一时段问候语
        '''
        if not self._init_ok:
            self._init_json()
            
        self._greetings[meal.value[0]].append(greeting)
        self._save()

        return MessageSegment.text(f"{greeting} 已加入 {meal.value[1]} 问候~")
    
    def show_greetings(self, meal: Meals) -> MessageSegment:
        '''
            展示某一时段问候语并标号
            等待用户输入标号，调用 remove_greeting 删除
        '''
        if not self._init_ok:
            self._init_json()
            
        msg: MessageSegment = ""
        i: int = 1
        for greeting in self._greetings[meal.value[0]]:
            if i < len(self._greetings[meal.value[0]]):
                msg += MessageSegment.text(f"{i}-{greeting}\n")
            else:
                msg += MessageSegment.text(f"{i}-{greeting}")
                
            i += 1
        
        return msg
            
    def remove_greeting(self, meal: Meals, index: int) -> MessageSegment:
        '''
            删除某一时段问候语
        '''
        if not self._init_ok:
            self._init_json()
            
        if index > len(self._greetings[meal.value[0]]):
            return MessageSegment.text("输入序号不合法")
        else:
            greeting = self._greetings[meal.value[0]].pop(index-1)
            self._save()
        
        return MessageSegment.text(f"{greeting} 已从 {meal.value[1]} 问候中移除~")

    async def do_greeting(self, meal: Meals) -> None:
        bot = get_bot()
        msg = self.get_greeting(meal)
        if msg and len(list(eating_manager._greetings["groups_id"])) > 0:
            for gid in eating_manager._greetings["groups_id"]:
                if eating_manager._greetings["groups_id"].get(gid, False):
                    try:
                        await bot.send_group_msg(group_id=int(gid), message=msg)
                    except ActionFailed as e:
                        logger.warning(f"发送群 {gid} 失败：{e}")
    
    def get_greeting(self, meal: Meals) -> Union[MessageSegment, None]:
        '''
            干饭/摸鱼小助手: Get greeting, return None when empty
        '''
        if not self._init_ok:
            self._init_json()
            
        if meal.value[0] in self._greetings:
            if len(self._greetings.get(meal.value[0])) > 0:
                greetings: List[str] = self._greetings.get(meal.value[0])
                if meal.value[0] == "breakfast":
                    img_path: Path = what2eat_config.what2eat_path / "img" / "breakfast"
                    files: List[str] = [str(f) for f in img_path.iterdir() if f.is_file()]
                    image: Path = img_path / random.choice(files)
                    return MessageSegment.text(random.choice(greetings)) + MessageSegment.image(image)
                
                elif meal.value[0] == "snack":
                    img_path: Path = what2eat_config.what2eat_path / "img" / "snack"
                    files: List[str] = [str(f) for f in img_path.iterdir() if f.is_file()]
                    image: Path = img_path / random.choice(files)
                    return MessageSegment.text(random.choice(greetings)) + MessageSegment.image(image)
                else:
                    return MessageSegment.text(random.choice(greetings))
        
        return None

    def _save(self) -> None:
        '''
            保存数据
        '''
        with open(self._eating_json, 'w', encoding='utf-8') as f:
            json.dump(self._eating, f, ensure_ascii=False, indent=4)
        
        with open(self._greetings_json, 'w', encoding='utf-8') as f:
            json.dump(self._greetings, f, ensure_ascii=False, indent=4)

eating_manager = EatingManager()

__all__ = [
    eating_manager
]


from pathlib import Path
from typing import List, Dict, Union
from nonebot import logger
from pydantic.errors import PathError
from .config import what2eat_config, save_json
try:
    import ujson as json
except ModuleNotFoundError:
    import json
  
def do_compatible(new_eating_json: Path, new_greeting_json: Path) -> None:
    '''
        v0.3.0 is compatible with configure json file of version 0.2.x, but will be deprecated in next version
        Rename old file(data.json) to eating.json, greating.json(that's a wrong word) to greetings.json
        Replace old keys(eating) with new keys(count)
    '''
    logger.warning("v0.3.0 is compatible with configure json file of version 0.2.x, but will be deprecated in next version")
    
    old_data_json: Path = what2eat_config.what2eat_path / "data.json"
    
    if not old_data_json.exists():
        logger.info(f"Old data.json doesn't exist, ignored: {old_data_json}")
    else:
        # Rename
        res = old_data_json.rename(new_eating_json)
        if not res.exists():
            logger.warning(f"Create json files of new version failed!")
            raise PathError
        else:
            with new_eating_json.open("r", encoding="utf-8") as f:
                # Change key value
                _f: Dict[str, Union[List[str], Dict[str, Union[Dict[str, List[int]], List[str]]]]] = json.load(f)
                try:
                    _f["count"] = _f.pop("eating")
                except KeyError as e:
                    logger.warning(f"Key error missing: {e}")
            
            save_json(new_eating_json, _f)
    
    old_greating_json: Path = what2eat_config.what2eat_path / "greating.json"
    
    if not old_greating_json.exists():
        logger.info(f"Old greating.json doesn't exist, ignored: {old_greating_json}")
    else:
        # Rename
        res = old_greating_json.rename(new_greeting_json)
        if not res.exists():
            logger.warning(f"Create json files of new version failed!")
            raise PathError
    
    logger.debug("Compatible work success!")

__all__ = [
    do_compatible
]


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot_plugin_what2eat']

package_data = \
{'': ['*'], 'nonebot_plugin_what2eat': ['resource/*']}

install_requires = \
['httpx>=0.23.0,<0.24.0',
 'nonebot-adapter-onebot>=2.0.0-beta.1,<3.0.0',
 'nonebot-plugin-apscheduler>=0.1.2,<0.2.0',
 'nonebot2>=2.0.0-beta.2,<3.0.0']

setup_kwargs = {
    'name': 'nonebot-plugin-what2eat',
    'version': '0.3.1',
    'description': 'Ask bot for what to eat today!',
    'long_description': '<div align="center">\n    <img width="200" src="starving_logo.gif" alt="logo">\n\n# What to Eat\n\n<!-- prettier-ignore-start -->\n<!-- markdownlint-disable-next-line MD036 -->\n_🍔🌮🍜🍮🍣🍻🍩 今天吃什么 🍩🍻🍣🍮🍜🌮🍔_\n<!-- prettier-ignore-end -->\n\n</div>\n\n<p align="center">\n  \n  <a href="https://github.com/MinatoAquaCrews/nonebot_plugin_what2eat/blob/beta/LICENSE">\n    <img src="https://img.shields.io/github/license/MinatoAquaCrews/nonebot_plugin_what2eat?color=blue">\n  </a>\n  \n  <a href="https://github.com/nonebot/nonebot2">\n    <img src="https://img.shields.io/badge/nonebot2-2.0.0beta.2+-green">\n  </a>\n  \n  <a href="https://github.com/MinatoAquaCrews/nonebot_plugin_what2eat/releases/tag/v0.3.1">\n    <img src="https://img.shields.io/github/v/release/MinatoAquaCrews/nonebot_plugin_what2eat?color=orange">\n  </a>\n\n  <a href="https://www.codefactor.io/repository/github/MinatoAquaCrews/nonebot_plugin_what2eat">\n    <img src="https://img.shields.io/codefactor/grade/github/MinatoAquaCrews/nonebot_plugin_what2eat/beta?color=red">\n  </a>\n  \n</p>\n\n</p>\n\n## 版本\n\nv0.3.1\n\n⚠ 适配nonebot2-2.0.0beta.2+\n\n[更新日志](https://github.com/MinatoAquaCrews/nonebot_plugin_what2eat/releases/tag/v0.3.1)\n\n## 安装\n\n1. 通过`pip`或`nb`安装；\n\n2. 数据默认位于`./resource`下`eating.json`与`greetings.json`，可通过设置`env`下`WHAT2EAT_PATH`更改；基础菜单、群特色菜单及群友询问Bot次数会记录在该文件中：\n\n    ```python\n    WHAT2EAT_PATH="your-path-to-resource"\n    ```\n\n## 功能\n\n1. 选择恐惧症？让Bot建议你今天吃什么！\n\n2. ⚠ 插件配置：\n   \n    ``` python\n    WHAT2EAT_PATH="your-path-to-resource"         # 资源路径\n    USE_PRESET_MENU=false                         # 是否从repo中下载预置基础菜单，默认为False\n    USE_PRESET_GREETINGS=false                    # 是否从repo中下载预置问候语，默认为False\n    EATING_LIMIT=5                                # 每个时段吃什么的次数上限，默认5次；每日6点、11点、17点、22点自动刷新\n    GREETING_GROUPS_ID=["123456789", "987654321"] # 默认开启小助手群组，或{"123456789", "987654321"}\n    SUPERUSERS={"12345678"}                       # 同nonebot超管配置\n    ```\n\n3. 群管理可自行添加或移除群特色菜单（位于`eating.json`下`[group_food][group_id]`）；超管可添加或移除基础菜单（`[basic_food]`）；\n\n4. 各群特色菜单相互独立；各群每个时间段询问Bot建议次数独立；Bot会综合各群菜单+基础菜单给出建议；\n\n5. 吃饭小助手：每天7、12、15、18、22点群发问候语提醒群友吃饭/摸鱼/下班，`GREETING_GROUPS_ID`设置常开的群号列表（或集合），每次启动插件时将置`True`，形如：\n\n    ```python\n    GREETING_GROUPS_ID=["123456789", "987654321"] # 名字长防止与其他插件配置名相同\n    ```\n\n    ⚠ 群吃饭小助手启用配置存于`greetings.json`的`groups_id`字段\n\n6. 初次使用该插件时，若不存在`eating.json`与`greetings.json`文件，设置`USE_PRESET_MENU`及`USE_PRESET_GREETINGS`可从仓库中尝试下载；当下载失败会写入初始键值。\n\n    ```python\n    USE_PRESET_MENU=false\n    USE_PRESET_GREETINGS=false\n    ```\n\n    ⚠ 若存在`eating.json`与`greetings.json`，不会从仓库中下载，仅检查是否缺少必要的键值，并补写初始键值\n\n    ⚠ 从仓库下载会**覆写**原有配置！建议老用户按需开启从仓库下载设置\n\n    ⚠ `v0.3.1`版本会兼容`v0.2.x`，在插件启动时启用自动兼容：检测`v0.2.x`的配置文件，将其自动改为当前版本的配置文件及键值。但当原`v0.2.x`的配置文件不存在时则忽略。后续版本将弃用兼容，请注意\n\n## 命令\n\n1. 吃什么：今天吃什么、中午吃啥、今晚吃啥、中午吃什么、晚上吃啥、晚上吃什么、夜宵吃啥……\n\n2. [管理员或超管] 添加或移除群菜名：[添加/移除 菜名]；\n\n3. 查看群菜单：[菜单/群菜单/查看菜单]；\n\n4. [超管] 添加至基础菜单：[加菜 菜名]；\n\n5. 查看基础菜单：[基础菜单]；\n\n6. [管理员或超管] 开启/关闭吃饭小助手：[开启/启用/关闭/禁用小助手]；\n\n7. [管理员或超管] 添加/删除吃饭小助手问候语：[添加/删除/移除问候 时段 问候语]；\n\n    ⚠ 添加/移除问候操作可一步步进行，或一次性输入两或三个命令；可中途取消操作\n\n## 效果\n\n1. 示例1\n\n    Q：今晚吃什么\n\n    A：建议肯德基\n\n    （吃什么*5）\n\n    Q：今晚吃什么\n\n    A：你今天已经吃得够多了！\n\n    Q：群菜单\n\n    A：\n\n    ---群特色菜单---\n\n    alpha\n\n    beta\n\n    gamma\n\n2. 示例2\n\n    [群管] Q：添加 派蒙\n\n    A：派蒙 已加入群特色菜单~\n\n    [超管] Q：加菜 东方馅挂炒饭\n\n    A：东方馅挂炒饭 已加入基础菜单~\n\n    [群管] Q：移除 东方馅挂炒饭\n\n    A：东方馅挂炒饭 在基础菜单中，非超管不可操作哦~\n\n3. 示例3\n\n    [群管] Q：添加问候\n\n    A：请输入添加问候语的时段，可选：早餐/午餐/摸鱼/晚餐/夜宵，输入取消以取消操作\n\n    [群管] Q：摸鱼\n\n    A：请输入添加的问候语，输入取消以取消操作\n\n    [群管] Q：你好\n\n    A：你好 已加入 摸鱼问候~\n\n## 本插件改自\n\n[HoshinoBot-whattoeat](https://github.com/pcrbot/whattoeat)\n\n部分菜名参考[程序员做饭指南](https://github.com/Anduin2017/HowToCook)',
    'author': 'KafCoppelia',
    'author_email': 'k740677208@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.3,<4.0.0',
}


setup(**setup_kwargs)
